/*
 * Copyright 2024 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#pragma once

#include <chrono>
#include <condition_variable>
#include <mutex>
#include <thread>

#include <string>
#include <kylin-ai/ai-engine/large-model/endsidenlpengine.h>

class NlpServer;

namespace ai_engine::lm::nlp {

class OndeviceNlpEngine : public AbstractEndSideNlpEngine {
public:
    OndeviceNlpEngine();
    ~OndeviceNlpEngine();
    std::string engineName() override { return "ondevice"; }
    void setChatResultCallback(ChatResultCallback callback) override;
    void setContextSize(int size) override;
    void clearContext() override;
    virtual bool chat(const std::string &message,
                      ai_engine::lm::EngineError &error) override;
    void stopChat() override;
    void onChatResult(const std::string &result);

    virtual bool initChatModule(ai_engine::lm::EngineError &error) override;
    virtual bool destroyChatModule(ai_engine::lm::EngineError &error) override;

    uint taskId() const { return taskId_; }

private:
    void releaseTimerCallback();

private:
    uint taskId_ = 0;
    int slotId_ = -1;
    uint sessionId_ = 0;
    bool inited_ = false;
    bool chatStopped_ = false;
    std::mutex releaseTimerMutex_;
    std::condition_variable releaseTimerCondition_;
    NlpServer *nlpServer_ = nullptr;
    ChatResultCallback chatResultCallback_ = nullptr;
};

} // namespace ai_engine::lm::nlp
